/* Programme de lecture d'une sonde de température LM35
*  Sauvegarde des données sur une carte SD
*  Code réalisé par Renan WAROUX pour le camps "Laboratoire du futur!"
*  Pour Objectif Sciences International
*  Date: 28/07/2011
*/

#include <SD.h>
#include <Wire.h>
#define NB_MAX_SAVE 60000 //Nombre maximal de sauvegarde par fichier

File myFile;

int cspin = 10; //Pin 10 pour le shield SD
int pin = 0; // analog pin
int tempc = 0; //variables de température
int samples[8]; // variables to make a better precision
int maxi = -100,mini = 100; // to start max/min temperature
int i;
long compteur = 0; //Un compteur pour compter le nombre de sauvegarde effectué
int indice_save = 1; //Un indice pour le numéro de la sauvegarde
char save_name[12]; //Nom de la sauvegarde

//Pour l'horloge
byte secs,mins,hrs,jour,date,mois,annee;
  

void setup()
{
  Wire.begin();//Initialisation de l'arduino en mode maitre
  Serial.begin(9600);//Début de la communication série
  //MiseALHeure(); //Mise à l'heure automatique, à décommenter pour mettre l'heure à jour

Serial.print("Initializing SD card...");
// On the Ethernet Shield, CS is pin 4. It's set as an output by default.
// Note that even if it's not used as the CS pin, the hardware SS pin 
// (10 on most Arduino boards, 53 on the Mega) must be left as an output 
// or the SD library functions will not work. 
pinMode(10, OUTPUT);
   
if (!SD.begin(cspin))
  {
    Serial.println("initialization failed!");
    return;
  }
  
  Serial.println("initialization done.");
  
   //On sauvegarde les données
  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  
  // //On détermine le nom de la sauvegarde en fonction du numéro de la sauvegarde actuel
  if(indice_save > 0)
  {
    strcpy(save_name ,"save000.csv");
    if(indice_save < 9)
    {
      save_name[6] = indice_save+'0';
    }
    else if(indice_save < 99)
    {
      save_name[6] = indice_save%10 + '0';//On extrant les unités
      save_name[5] = (indice_save - indice_save%10)/10 + '0';//On extrait les dizaines
    }
    else if(indice_save < 999)
    {
      save_name[6] = indice_save%10 + '0';//On extrant les unités
      save_name[5] = (indice_save%100 - indice_save%10)/10 +'0';//On extrait les dizaines
      save_name[4] = (indice_save - indice_save%100)/100 +'0';//On extrait les centaines
    }
  }
  
  //On ouvre la sauvegarde
  myFile = SD.open(save_name, FILE_WRITE);
  
  // if the file opened okay, write to it:
  if (myFile) 
  {
    Serial.print("Writing to ");
    Serial.print(save_name);	
    // close the file:
    myFile.close();
    Serial.println(" done.");
  } 
  else
  {
    // if the file didn't open, print an error:
    Serial.print("error opening ");
    Serial.println(save_name);
    
  } 

}//Fin de setup

void loop()
{
  
  //On récupère la température
  getTemperature();
  //Horloge
  //Initialisation du pointeur de registre
  Wire.beginTransmission(0x68); //Condition de départ
  Wire.send(0); //Ecriture du pointeur de registre
  Wire.endTransmission();//Condition d'arret
  
  //Affichage données
  affichage_donnees();
  
  //Sauvegarde des données
  writeToSDCard();  
  //On attends 5 s
  delay(5000);

  tempc = 0;
  
  //On incrémente le compteur
  if( compteur < NB_MAX_SAVE)
  {
    compteur++;
  }
  else //Sinon, on remet le compteur à 0 et on incrémente le numéro de la sauvegarde
  {
    compteur = 0;
    indice_save++;
  }
}//Fin de la fonction Loop

void affichage_donnees()
{
  //On lit les données depuis l'horloge interne
  readFromClock();
  
  //Mise en forme et affichage des données
  //Mise en forme de l'heure
  Serial.print("Heure ");
  if( hrs < 10) Serial.print("0");
  Serial.print(hrs,HEX);
  Serial.print(":");
  if( mins < 10) Serial.print("0");
  Serial.print(mins,HEX);
  Serial.print(":");
  if( secs < 10) Serial.print("0");
  Serial.print(secs,HEX);
  Serial.print(" Date ");
  
  //Mise en forme de la date
  if( date < 10) Serial.print("0");
  Serial.print(date,HEX);
  Serial.print("-");
  
  if( mois < 10) Serial.print("0");
  Serial.print(mois, HEX);
  Serial.print("-");
  Serial.print("20"); //Ajout de 20 pour les années 20
  
  if(annee < 10 ) Serial.print("0");
  Serial.println(annee,HEX);
  
  //Affichage des données de température
  Serial.print("Temperature: ");
  Serial.print(tempc,DEC);
  
  //Affichage des données sur le port série
  Serial.print(" C| Max: ");
  Serial.print(maxi,DEC);
  Serial.print(" C| Min ");
  Serial.print(mini,DEC);
  Serial.println(" C");
  Serial.println();
 
}

void readFromClock()
{
   //Lecture des 7 octets contenant la date et l'heure
  Wire.requestFrom(0x68, 7);
  secs = Wire.receive();
  mins = Wire.receive();
  hrs = Wire.receive();
  jour = Wire.receive();
  date = Wire.receive();
  mois = Wire.receive();
  annee = Wire.receive();
  
}
 /*
//A décommenter pour mettre l'heure à jour
//Fonction de mise à l'heure simplifiée
void MiseALHeure()
{
  Wire.beginTransmission(0x68); //Condition de démarrage
  Wire.send(0); //Initialisation du pointeur de registres
  Wire.send(0x00); //Les secondes
  Wire.send(0x02); //Les minutes
  Wire.send(0x80 | 0x19); //L'heure au format 24H
  Wire.send(0x01);  //Jour de la semaine 01-07
  Wire.send(0x25); //Jour dans le mois 1-31
  Wire.send(0x07); //Mois de l'année 1-12
  Wire.send(0x11); //Année
  Wire.send(0x10); //Gestion de la sortie SQWOUT
  Wire.endTransmission(); //Condition d'arret
}*/

void getTemperature()
{
  //Lecture de la sonde de température
  for(i = 0;i <= 7;i++)// gets 8 samples of temperature
  { 
  samples[i] = ( 5.0 * analogRead(pin) * 100.0) / 1024.0;
  tempc = tempc + samples[i];
  }
  tempc = tempc/8.0; // better precision
  //tempf = (tempc * 9)/ 5 + 32; // converts to fahrenheit
  if(tempc > maxi) {maxi = tempc;} // set max temperature
  if(tempc < mini) {mini = tempc;} // set min temperature
    
}

//Cette fonction s'occupe d'ouvrir un fichier de sauvegarde pour sauver les mesures faites
void writeToSDCard()
{
  //On détermine le nom de la sauvegarde en fonction du numéro de la sauvegarde actuel
   if(indice_save > 0)
  {
    strcpy(save_name ,"save000.csv");
    if(indice_save < 9)
    {
      save_name[6] = indice_save+'0';
    }
    else if(indice_save < 99)
    {
      save_name[6] = indice_save%10 + '0';//On extrant les unités
      save_name[5] = (indice_save - indice_save%10)/10 + '0';//On extrait les dizaines
    }
    else if(indice_save < 999)
    {
      save_name[6] = indice_save%10 + '0';//On extrant les unités
      save_name[5] = (indice_save%100 - indice_save%10)/10 +'0';//On extrait les dizaines
      save_name[4] = (indice_save - indice_save%100)/100 +'0';//On extrait les centaines
    }
  }
  
  //On ouvre le fichier de sauvegarde
  myFile = SD.open(save_name, FILE_WRITE);
 
  // if the file opened okay, write to it:
  if (myFile) 
  {
    //Mise en forme et affichage des données
    if( hrs < 10) myFile.print("0");
    myFile.print(hrs,HEX);
    myFile.print(":");
    if( mins < 10) myFile.print("0");
    myFile.print(mins,HEX);
    myFile.print(":");
    if( secs < 10) myFile.print("0");
    myFile.print(secs,HEX);
    myFile.print(",");
    
    if( date < 10) myFile.print("0");
    myFile.print(date,HEX);
    myFile.print("/");
    
    if( mois < 10)myFile.print("0");
    myFile.print(mois, HEX);
    myFile.print("/");
    myFile.print("20"); //Ajout de 20 pour les années 20
    
    if(annee < 10 ) myFile.print("0");
    myFile.print(annee,HEX);
    
    //Affichage des données de température
    myFile.print(",");
    myFile.print(tempc,DEC);
   /* 
    myFile.print(",");
    myFile.print(maxi,DEC);
    myFile.print(",");
    myFile.print(mini,DEC); */
    myFile.println();	
    // close the file:
    myFile.close();
  } 
  else
  {
    // if the file didn't open, print an error:
    Serial.print("error opening ");
    Serial.println(save_name);
  }  
}
